@@ -429,9 +429,14 @@ class Node:
         """Returns the load-balancing penalty for this node."""
         if not self.available or not self.stats:
             return float("inf")
-
         return self.stats.penalty.total
 
+    async def penalty_with_region(self, region: str | None) -> float:
+        """The penalty for the node, with the region added in."""
+        if not region:
+            return self.penalty
+        return self.penalty + (1.1 ** (0.0025 * await self.region_distance(region)) * 500 - 500)
+
     def dispatch_event(self, event: Event) -> None:
         """|coro|
         Dispatches the given event to all registered hooks.
@@ -461,13 +466,20 @@ class Node:
     def __eq__(self, other):
         if isinstance(other, Node):
             return self.identifier == other.identifier
+        elif isinstance(other, NodeModel):
+            return self.host == other.yaml["server"]["address"] and self.port == other.yaml["server"]["port"]
         return NotImplemented
 
     def __ne__(self, other):
         if isinstance(other, Node):
             return self.identifier != other.identifier
+        elif isinstance(other, NodeModel):
+            return not (self.host == other.yaml["server"]["address"] and self.port == other.yaml["server"]["port"])
         return NotImplemented
 
+    def __hash__(self) -> int:
+        return hash((self.host, self.port))
+
     async def get_query_youtube_music(self, query: str, bypass_cache: bool = False) -> LavalinkResponseT:
         """|coro|
         Gets the query from YouTube music.
@@ -1246,7 +1258,7 @@ class Node:
         """
         coordinates = REGION_TO_COUNTRY_COORDINATE_MAPPING.get(region)
 
-        if not (coordinates or self.coordinates):
+        if not (coordinates and self.coordinates):
             return float("inf")
 
         return distance(*self.coordinates, *coordinates)
